package com.qiaoba.module.system.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qiaoba.api.system.entity.SysConfig;
import com.qiaoba.api.system.entity.param.SysConfigParam;
import com.qiaoba.auth.utils.SecurityUtil;
import com.qiaoba.common.base.constants.BaseConstant;
import com.qiaoba.common.base.constants.ConfigConstant;
import com.qiaoba.common.base.exceptions.ServiceException;
import com.qiaoba.common.database.entity.PageQuery;
import com.qiaoba.common.database.entity.TableDataInfo;
import com.qiaoba.common.redis.service.RedisService;
import com.qiaoba.module.system.mapper.SysConfigMapper;
import com.qiaoba.module.system.service.SysConfigService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 系统配置 服务层实现
 *
 * @author ailanyin
 * @version 1.0
 * @since 2023-04-23 20:33:43
 */
@Service
@RequiredArgsConstructor
public class SysConfigServiceImpl implements SysConfigService {

    private final SysConfigMapper sysConfigMapper;
    private final RedisService redisService;


    @Override
    public int insert(SysConfig sysConfig) {
        checkKeyIsExist(sysConfig);
        sysConfig.setCreateTime(new Date());
        sysConfig.setCreateUser(SecurityUtil.getLoginUsername());
        int result = sysConfigMapper.insert(sysConfig);
        if (result > BaseConstant.HANDLE_ERROR) {
            cacheConfig(sysConfig);
        }
        return result;
    }

    @Override
    public int updateById(SysConfig sysConfig) {
        checkKeyIsExist(sysConfig);
        sysConfig.setUpdateTime(new Date());
        sysConfig.setUpdateUser(SecurityUtil.getLoginUsername());
        int result = sysConfigMapper.updateById(sysConfig);
        if (result > BaseConstant.HANDLE_ERROR) {
            cacheConfig(sysConfig);
        }
        return result;
    }

    @Override
    public List<SysConfig> selectList(SysConfigParam param) {
        return sysConfigMapper.selectList(param2Wrapper(param));
    }

    @Override
    public SysConfig selectById(String configId) {
        return sysConfigMapper.selectById(configId);
    }

    @Override
    public int deleteById(String configId) {
        SysConfig sysConfig = selectById(configId);
        if (Objects.isNull(sysConfig)) {
            return BaseConstant.HANDLE_ERROR;
        }
        if (BaseConstant.YES.equals(sysConfig.getConfigType())) {
            throw new ServiceException("系统内置不允许删除！");
        }
        redisService.del(ConfigConstant.SYS_CONFIG_KEY_PREFIX + sysConfig.getConfigKey());
        return sysConfigMapper.deleteById(configId);
    }

    @Override
    public int deleteByIds(List<String> ids) {
        List<SysConfig> configs = sysConfigMapper.selectByIds(ids);
        if (CollUtil.isEmpty(configs)) {
            return BaseConstant.HANDLE_ERROR;
        }
        List<String> delKeys = new ArrayList<>();
        for (SysConfig config : configs) {
            if (BaseConstant.YES.equals(config.getConfigType())) {
                throw new ServiceException("系统内置不允许删除！");
            }
            delKeys.add(ConfigConstant.SYS_CONFIG_KEY_PREFIX + config.getConfigKey());
        }
        redisService.del(delKeys);
        return sysConfigMapper.deleteBatchIds(ids);
    }

    @Override
    public void resetConfigCache() {
        List<SysConfig> list = selectList(new SysConfigParam());
        for (SysConfig sysConfig : list) {
            cacheConfig(sysConfig);
        }
    }

    @Override
    public String selectConfigByKey(String configKey) {
        return redisService.getObject(ConfigConstant.SYS_CONFIG_KEY_PREFIX + configKey, String.class);
    }

    private QueryWrapper<SysConfig> param2Wrapper(SysConfigParam param) {
        QueryWrapper<SysConfig> wrapper = new QueryWrapper<>();
        wrapper.lambda()
                .like(StrUtil.isNotBlank(param.getConfigName()), SysConfig::getConfigName, param.getConfigName())
                .like(StrUtil.isNotBlank(param.getConfigKey()), SysConfig::getConfigKey, param.getConfigKey())
                .eq(StrUtil.isNotBlank(param.getConfigType()), SysConfig::getConfigType, param.getConfigType());
        return wrapper;
    }

    @Override
    public TableDataInfo<SysConfig> selectPageList(SysConfigParam param, PageQuery pageQuery) {
        return TableDataInfo.build(sysConfigMapper.selectPage(pageQuery.build(), param2Wrapper(param)));
    }

    private void checkKeyIsExist(SysConfig sysConfig) {
        if (StrUtil.isNotBlank(sysConfigMapper.checkKeyIsExist(sysConfig))) {
            throw new ServiceException("参数Key已存在, 操作失败！");
        }
    }

    private void cacheConfig(SysConfig sysConfig) {
        redisService.set(ConfigConstant.SYS_CONFIG_KEY_PREFIX + sysConfig.getConfigKey(), sysConfig.getConfigValue());
    }
}
